
var exportSprites = function(sprites, writePivotPoints)
{
    var frames = {};
    for (var i = 0; i < sprites.length; i++)
    {
        var s = sprites[i];
        var spriteInfo = {
            size: { w:s.untrimmedSize.width, h:s.untrimmedSize.height },
            frame: { x:s.frameRect.x, y:s.frameRect.y, w:s.frameRect.width, h:s.frameRect.height },
            offset: { x:s.sourceRect.x, y:s.sourceRect.y },
            rotated: s.rotated
        };

        if (s.pivotPointNorm && writePivotPoints)
        {
            spriteInfo.pivot = { x:s.pivotPointNorm.x, y:s.pivotPointNorm.y };
        }
        if (s.scale9Enabled)
        {
            spriteInfo.scale9 = {
                x:s.scale9Borders.x,
                y:s.scale9Borders.y,
                w:s.scale9Borders.width,
                h:s.scale9Borders.height
            };
        }
        frames[s.name] = spriteInfo;
    }
    return frames;
}


var exportTextures = function(root)
{
    var textures = root.allResults[root.variantIndex].textures;
    var sheets = [];

    for (var i = 0; i < textures.length; i++)
    {
        var t = textures[i];
        var textureInfo = {
            texture: t.fullName,
            normalMap: (t.normalMapFileName === "") ? undefined : t.normalMapFileName,
            frames: exportSprites(t.allSprites, root.settings.writePivotPoints)
        };
        sheets.push(textureInfo);
    }
    return sheets;
}


var exportData = function(root)
{
    var doc = {
        textures: exportTextures(root),
        meta: {
            app: "https://www.codeandweb.com/texturepacker",
            exporter: "gamemaker",
            version: "1.0",
            smartupdate: root.smartUpdateKey
        },
    };
    var str = JSON.stringify(doc, null, "\t");

    // remove newlines from objects like { "w": 2044, "h": 1536 } and { "x": 1, "y": 1, "w": 2044, "h": 1536 }
    str = str.replace(/\{\s+"\w": \d+(,\s+"\w": \d+)*\s+\}/g, match => match.replace(/\s+/g, ' '));
    return str;
}
exportData.filterName = "exportData";
Library.addFilter("exportData");

